<?php

namespace Drupal\gooey_ai;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactory;
use Drupal\key\KeyRepository;
use GuzzleHttp\Client;

/**
 * Gooey AI API creator.
 */
class GooeyAiClient {

  /**
   * API Key.
   */
  private string $apiKey;

  /**
   * The base host.
   */
  private string $baseHost = 'https://api.gooey.ai/v2/';

  /**
   * Constructs a new Gooey AI object.
   *
   * @param \GuzzleHttp\Client $client
   *   Http client.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The config factory.
   * @param \Drupal\key\KeyRepository $keyRepository
   *   The key repository.
   */
  public function __construct(
    protected Client $client,
    protected ConfigFactory $configFactory,
    protected KeyRepository $keyRepository,
  ) {
    $this->invokeApiKey();
  }

  /**
   * Get the api key.
   */
  protected function invokeApiKey() {
    $config = $this->configFactory->get('gooey_ai.settings');
    $this->apiKey = $this->keyRepository->getKey($config->get('api_key'))->getKeyValue();
  }

  /**
   * Set API key.
   *
   * @param string $apiKey
   *   The api key.
   */
  public function setApiKey($apiKey) {
    $this->apiKey = $apiKey;
  }

  /**
   * Make a qr call.
   *
   * @param string $qr_code_data
   *   The qr code data, like a link or text.
   * @param string $prompt
   *   The prompt on how to generate the qr code.
   * @param string $use_url_shortener
   *   Whether to use the url shortener.
   * @param array $extra_options
   *   Extra options.
   *
   * @return array
   *   The response.
   */
  public function makeQrCode(string $qr_code_data, string $prompt, string $use_url_shortener = 'false', array $extra_options = []): array {
    $data = [
      'qr_code_data' => $qr_code_data,
      'text_prompt' => $prompt,
      'use_url_shortener' => $use_url_shortener,
    ];

    $data = array_merge($data, $extra_options);

    return Json::decode($this->makeRequest('art-qr-code', [], 'POST', $data));
  }

  /**
   * Make gooey ai call.
   *
   * @param string $path
   *   The path.
   * @param array $query_string
   *   The query string.
   * @param string $method
   *   The method.
   * @param array $body
   *   Data to attach if POST/PUT/PATCH.
   * @param array $options
   *   Extra headers.
   *
   * @return string|object
   *   The return response.
   */
  protected function makeRequest(string $path, array $query_string = [], string $method = 'GET', array $body = [], array $options = []) {
    if (!$this->apiKey) {
      throw new \Exception('No api key set.');
    }
    // Don't wait to long.
    $options['connect_timeout'] = 120;
    $options['read_timeout'] = 120;
    $options['timeout'] = 120;

    // JSON unless its multipart.
    if (empty($options['multipart'])) {
      $options['headers']['Content-Type'] = 'application/json';
    }

    // Credentials.
    $options['headers']['authorization'] = 'Bearer ' . $this->apiKey;
    if ($body) {
      $options['body'] = json_encode($body);
    }

    $new_url = rtrim($this->baseHost, '/') . '/' . $path;
    $new_url .= count($query_string) ? '?' . http_build_query($query_string) : '';

    $res = $this->client->request($method, $new_url, $options);

    return $res->getBody();
  }

}
