<?php

namespace Drupal\gooey_ai\Plugin\AiAutomatorType;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai_automators\Attribute\AiAutomatorType;
use Drupal\ai_automators\PluginBaseClasses\ExternalBase;
use Drupal\ai_automators\PluginInterfaces\AiAutomatorTypeInterface;
use Drupal\ai_automators\Traits\FileHelperTrait;
use Drupal\gooey_ai\GooeyAiClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The rules for an an address.
 */
#[AiAutomatorType(
  id: 'gooey_ai_qr_code',
  label: new TranslatableMarkup('Gooey AI: QR Code from Link'),
  field_rule: 'image',
  target: 'file',
)]
class QrCode extends ExternalBase implements AiAutomatorTypeInterface, ContainerFactoryPluginInterface {

  use FileHelperTrait;

  /**
   * {@inheritDoc}
   */
  public $title = 'Gooey AI: QR Code from Link';

  /**
   * Construct an image field.
   *
   * @param array $configuration
   *   Inherited configuration.
   * @param string $plugin_id
   *   Inherited plugin id.
   * @param mixed $plugin_definition
   *   Inherited plugin definition.
   * @param \Drupal\gooey_ai\GooeyAiClient $gooeyAiClient
   *   The Gooey AI client.
   */
  final public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected GooeyAiClient $gooeyAiClient,
  ) {
  }

  /**
   * {@inheritDoc}
   */
  final public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('gooey_ai.api'),
    );
  }

  /**
   * {@inheritDoc}
   */
  public function allowedInputs() {
    return [
      'link',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function helpText() {
    return $this->t("Creates a QR code from a link + a prompt.");
  }

  /**
   * {@inheritDoc}
   */
  public function placeholderText() {
    return "A man walking into a bar.";
  }

  /**
   * {@inheritDoc}
   */
  public function generate(ContentEntityInterface $entity, FieldDefinitionInterface $fieldDefinition, array $automatorConfig) {
    $prompts = parent::generate($entity, $fieldDefinition, $automatorConfig);
    $images = [];
    foreach ($prompts as $prompt) {
      foreach ($entity->get($automatorConfig['base_field'])->getValue() as $item) {
        if (!empty($item['uri'])) {
          $response = $this->gooeyAiClient->makeQrCode($item['uri'], $prompt);
          if (isset($response['output']['output_images'][0])) {
            $images[] = [
              'filename' => basename($response['output']['output_images'][0]),
              'binary' => file_get_contents($response['output']['output_images'][0]),
            ];
          }
        }
      }
    }
    return $images;
  }

  /**
   * {@inheritDoc}
   */
  public function verifyValue(ContentEntityInterface $entity, $value, FieldDefinitionInterface $fieldDefinition, array $automatorConfig) {
    if (!isset($value['filename'])) {
      return FALSE;
    }
    // Detect if binary.
    return preg_match('~[^\x20-\x7E\t\r\n]~', $value['binary']) > 0;
  }

  /**
   * {@inheritDoc}
   */
  public function storeValues(ContentEntityInterface $entity, array $values, FieldDefinitionInterface $fieldDefinition, array $automatorConfig) {
    $images = [];
    foreach ($values as $value) {
      $fileHelper = $this->getFileHelper();
      $path = $fileHelper->createFilePathFromFieldConfig($value['filename'], $fieldDefinition, $entity);
      $images[] = $fileHelper->generateImageMetaDataFromBinary($value['binary'], $path);
    }
    // Then set the value.
    $entity->set($fieldDefinition->getName(), $images);
    return TRUE;
  }

}
